#!/bin/bash
# k8s_minio_migrate.sh

set -euo pipefail

# Array of required environment variables
required_vars=("AWS_CONNECTION_STRING" "AWS_ACCESS_KEY" "AWS_SECRET_KEY" "AVATARS_AWS_CONNECTION_STRING" "AVATARS_AWS_ACCESS_KEY" "AVATARS_AWS_SECRET_KEY")

# Function to check if variables are set and not empty
check_required_variables() {
  local unset_vars=()
  for var in "${required_vars[@]}"; do
    if [[ -z "${!var:-}" ]]; then
      unset_vars+=("$var")
    fi
  done

  if [[ ${#unset_vars[@]} -gt 0 ]]; then
    echo "Please set the following environment variables: ${unset_vars[*]}"
    exit 1
  fi
}

# Check for required dependencies
command_check() {
  local required_command="$1"
  command -v "$required_command" >/dev/null 2>&1 || {
    echo "'$required_command' command not found. Make sure it's installed and in the PATH."
    exit 1
  }
}

# Validate input
validate_input() {
  if [ $# -ne 1 ]; then
    echo "Usage: $0 <namespace>"
    exit 1
  fi
}

# Validate the existence of the specified namespace
validate_namespace() {
    local namespace="$1"

    # Check if the provided namespace exists
    if ! kubectl get namespace "$namespace" &> /dev/null; then
        echo "Namespace '$namespace' not found."
        exit 1
    fi
}

# Compatibility for Windows
pwd_compatible() {
  if [[ "$(expr substr $(uname -s) 1 5)" == "MINGW" ]]; then
    work_dir=$(pwd -W)
  else
    work_dir=$(pwd)
  fi
}

# Function to stop deployments
stop_deployments() {
  local s_deployments=("$@")

  for deployment in "${s_deployments[@]}"; do
    kubectl scale deployment "$deployment" -n "$namespace" --replicas 0 > /dev/null 2>&1
  done
  for deployment in "${s_deployments[@]}"; do
    while ! [ -z "$(kubectl get deployment $deployment -n $namespace -o=jsonpath='{.status.readyReplicas}')" ]; do
      sleep 1
    done
  done
}

# Function to update deployments image
update_deployments() {
  local image="$1"
  shift
  local u_deployments=("$@")
  for deployment in "${u_deployments[@]}"; do
    echo "Updating deployment ${deployment} with image ${image}..."
    kubectl scale deployment "$deployment" -n "$namespace"  --replicas 1 > /dev/null 2>&1
    local container=$(kubectl get deployment $deployment -n $namespace -o=jsonpath='{.spec.template.spec.containers[0].name}')
    kubectl set image deployment/$deployment $container=$image -n $namespace
  done

  for deployment in "${u_deployments[@]}"; do
    local counter=0
    local timeout=180
    while [[ -z "$(kubectl get deployment "$deployment" -n "$namespace" -o=jsonpath='{.status.readyReplicas}')" ]]; do
      sleep 1
      counter+=1
      if [[ "$counter" == "$timeout" ]]; then
        echo "Deployment '$deployment' is taking longer than ${timeout}s to become ready."
        break
      fi
    done
  done
}

# Function to run k8s job and track its completion
run_job() {
  local job_name="$1"
  local job_path="$2"
  envsubst < "${job_path}/job.yaml" > ./tmp.yaml
  kubectl apply -f "${job_path}/configmap.yaml" -n "$namespace" > /dev/null 2>&1
  kubectl apply -f ./tmp.yaml -n "$namespace" > /dev/null 2>&1

  # Wait for the job to complete or fail using the loop with sleep
  while true; do
    # Get the name of the pod associated with the job
    local pod_name=$(kubectl get pods -n "$namespace" -l job-name="$job_name" -o jsonpath='{.items[0].metadata.name}')
    local succeeded_status=$(kubectl logs "$pod_name" -n "$namespace" | grep -c "Script finished.")
    local failed_status=$(kubectl get job "$job_name" -n "$namespace" -o=jsonpath='{.status.failed}')
    
    if [ "$succeeded_status" = "1" ]; then
      echo "Job '$job_name' successful."
      kubectl delete -f ./tmp.yaml -n "$namespace" 2>&1
      kubectl delete -f "${job_path}/configmap.yaml" -n "$namespace" > /dev/null 2>&1
      rm -f ./tmp.yaml
      break
    elif [ "$failed_status" = "1" ]; then
      echo "Job '$job_name' failed. Displaying logs:"
      # Display the logs of the failed pod
      kubectl logs "$pod_name" -n "$namespace"
      kubectl delete -f ./tmp.yaml -n "$namespace" 2>&1
      kubectl delete -f "${job_path}/configmap.yaml" -n "$namespace" > /dev/null 2>&1
      rm -f ./tmp.yaml
      exit 1
    fi
    sleep 1
  done
}

# Delete PersistentVolumeClaims, saving configs for recreation
delete_pvc() {
  local d_deployments=("minio" "avatars-minio")
  echo "Deleting PVC for ${d_deployments[@]}"
  mkdir -p "$backup_dir/pvc_configs"
  local pvc_configs_dir="$backup_dir/pvc_configs"
  local pvc_name
  for deployment in "${d_deployments[@]}"
  do
    pvc_name="${deployment}-data-pv-claim"
    check=$(kubectl get pvc -n $namespace $pvc_name --no-headers 2>/dev/null | wc -l)
    if [ $check -ne 1 ]; then
      echo "Error! No PVC '$pvc_name' found."
      # exit 1
    else
      # Save the PVC YAML configuration to a file
      kubectl get pvc $pvc_name -n "$namespace" -o yaml > "$pvc_configs_dir/${pvc_name}.yaml"
      # Remove kubernetes-generated fields
      trim_pvc_yaml "$pvc_configs_dir/${pvc_name}.yaml"
      echo "Saved PVC configuration for '$pvc_name' to '$pvc_configs_dir/${pvc_name}.yaml'."
    fi
  done
  stop_deployments "${d_deployments[@]}"
  # Delete PVCs
  kubectl delete -f "$backup_dir/pvc_configs/"
}

trim_pvc_yaml () {
  local file="$1"
  yq eval 'del(.metadata.annotations) | del(.metadata.creationTimestamp) | del(.metadata.finalizers) | del(.metadata.resourceVersion) | del(.metadata.uid) | del(.spec.volumeName) | del(.status)' "$file" > "$file.tmp"
  mv "$file.tmp" "$file"
}

# Recreate PersistentVolumeClaims using saved configs from 'delete_pvc()'
recreate_pvc() {
  local pvc_configs_dir="$backup_dir/pvc_configs"
  local pvc_files=("$pvc_configs_dir"/*.yaml)
  if [ ${#pvc_files[@]} -eq 0 ]; then
    echo "No PVC configurations found in the backup directory: $pvc_configs_dir"
    return 1
  fi

  echo "Recreating PVCs from the backup directory: $pvc_configs_dir"
  for pvc_file in "${pvc_files[@]}"; do
    # Extract the PVC name from the filename (remove the directory and extension)
    pvc_name=$(basename "$pvc_file" .yaml)

    # Recreate the PVC from the saved YAML configuration
    kubectl apply -f "$pvc_file" > /dev/null 2>&1

    # Wait until the PVC is created
    while [ -z "$(kubectl get pvc "$pvc_name" -n "$namespace" -o 'jsonpath={.status.phase}')" ]; do
      sleep 1
    done

    echo "Recreated PVC '$pvc_name'."
  done

  echo "PVC recreation complete."
}

import_minio() { 
  export host_path="$1"
  export USER_ID="$(echo $UID)"
  run_job "mc-import-job" "./jobs/minio/import"
}

export_minio() {
  export host_path="$1"
  run_job "mc-export-job" "./jobs/minio/export"
}

main() {
  namespace="$1"
  validate_namespace "$namespace"

  # Create local buffer
  pwd_compatible
  export backup_dir="$work_dir/temp"
  rm -rf "$backup_dir" || true
  mkdir -p "$backup_dir"
  
  # Stop all the Test IT deployments except minio servers
  local deployments=("frontend" "webapi" "rabbitmq" "rabbitmqconsumer" "auth" "ldapwebapi" "auth-cache" "license-service" "avatars-api" "influxdb" "transfer-service")
  stop_deployments "${deployments[@]}"

  # Import data and configs from buckets
  import_minio "$backup_dir"

  # Clear PVCs
  local buckets=("minio" "avatars-minio")
  stop_deployments "${buckets[@]}"
  delete_pvc #"${buckets[@]}"
  recreate_pvc

  # Update Minio deployments
  local new_image="minio/minio:RELEASE.2023-04-20T17-56-55Z"
  update_deployments "$new_image" "${buckets[@]}"

  # Export data and configs from local buffer
  export_minio "$backup_dir"

  # Cleanup
  rm -rf "$backup_dir"
  
  echo "Migration successful!"
}

# Validations before the script
check_required_variables
validate_input "$@"
command_check "kubectl"
command_check "yq"

main "$@"