#!/bin/bash
# k8s_restore.sh

set -euo pipefail

# Check for required environment variables
postgres_vars=("POSTGRES_HOST" "POSTGRES_PORT" "POSTGRES_USER" "POSTGRES_PASSWORD")
minio_vars=("AWS_CONNECTION_STRING" "AWS_ACCESS_KEY" "AWS_SECRET_KEY")

# Function to check if variables are set and not empty
check_required_variables() {
  local id="$1"
  shift
  local target_vars=("$@")
  local unset_vars=()
  for var in "${target_vars[@]}"; do
    if [[ -z "${!var:-}" ]]; then
      unset_vars+=("$var")
    fi
  done

  if [[ ${#unset_vars[@]} -gt 0 ]]; then
    echo "Please set the following environment variables in '$id' : ${unset_vars[*]}"
    exit 1
  fi
}
# Check for required dependencies
command_check() {
  local required_command="$1"
  command -v "$required_command" >/dev/null 2>&1 || {
    echo "'$required_command' command not found. Make sure it's installed and in the PATH."
    exit 1
  }
}

# Validate input
validate_input() {
  if [ $# -ne 2 ]; then
    echo "Usage: $0 <namespace> <backup_date>"
    exit 1
  fi
}

# Validate the existence of the specified namespace
validate_namespace() {
    local namespace="$1"

    # Check if the provided namespace exists
    if ! kubectl get namespace "$namespace" &> /dev/null; then
        echo "Namespace '$namespace' not found."
        exit 1
    fi
}

# Compatibility for Windows
pwd_compatible() {
  if [[ "$(expr substr $(uname -s) 1 5)" == "MINGW" ]]; then
    work_dir=$(pwd -W)
  else
    work_dir=$(pwd)
  fi
}

# Function to stop deployments
stop_deployments() {
  local s_deployments=("$@")

  for deployment in "${s_deployments[@]}"; do
    replicas["$deployment"]=$(kubectl get deployment "$deployment" -n "$namespace" -o=jsonpath='{.spec.replicas}')
    kubectl -n "$namespace" scale deployment "$deployment" --replicas 0 > /dev/null 2>&1
  done

  for deployment in "${s_deployments[@]}"; do
    while ! [ -z "$(kubectl get deployment $deployment -n $namespace -o=jsonpath='{.status.readyReplicas}')" ]; do
      sleep 1
    done
  done
}

# Function to start deployments
start_deployments() {
  local s_deployments=("$@")

  for deployment in "${s_deployments[@]}"; do
    kubectl scale deployment "$deployment" -n "$namespace" --replicas "${replicas["$deployment"]}" > /dev/null 2>&1
  done

  for deployment in "${s_deployments[@]}"; do
    local counter=0
    local timeout=180
    while [[ "$(kubectl get deployment "$deployment" -n "$namespace" -o=jsonpath='{.status.readyReplicas}')" != "${replicas["$deployment"]}" ]]; do
      sleep 1
      counter+=1
      if [[ "$counter" == "$timeout" ]]; then
        echo "Deployment '$deployment' is taking longer than ${timeout}s to become ready."
        break
      fi
    done
  done
}

# Function to check if the file exists
check_file_existence() {
  local filename="$1"
  if ! [ -f "$filename" ]; then
    echo "The file '$filename' is missing."
    echo "Please check your working directory and make sure the file is present."
    exit 1
  fi
}

# Function to check if a directory exists
check_directory_existence() {
  local directory_name="$1"
  if ! [ -d "$directory_name" ]; then
    echo "The directory '$directory_name' is missing."
    echo "Please check your working directory and make sure the directory is present."
    exit 1
  fi
}

# Function to find the statefulset with matching image
find_statefulset_with_image() {
    local image_partial="$1"
    local statefulset=$(kubectl get statefulsets -n $namespace -o=jsonpath='{range .items[*]}{@.metadata.name}{" "}{@.spec.template.spec.containers[*].image}{"\n"}{end}' | grep "$image_partial" | awk 'NR==1{print $1}')

    if [[ -z $statefulset ]]; then
        exit 1
    else
        echo "${statefulset}-0"
    fi
}

# Function to copy data to pod
copy_to_pod() {
  local pod="$1"
  local file="$2"
  local dest="$3"
 
  kubectl cp "${file}" "${pod}:${dest}" --retries 999 --no-preserve > /dev/null 2>&1
}

# Function to run k8s job and track its completion
run_job() {
  local job_name="$1"
  local job_path="$2"
  local arch="$3"
  envsubst < "${job_path}/job.yaml" > ./tmp.yaml
  kubectl apply -f "${job_path}/configmap.yaml" -n "$namespace" > /dev/null 2>&1
  kubectl apply -f ./tmp.yaml -n "$namespace" > /dev/null 2>&1
  # Get the name of the pod associated with the job
  local pod_name=""
  local status=0
  while [ $status -lt 1 ]; do
    pod_name=$(kubectl get pods -n "$namespace" -l job-name="$job_name" -o jsonpath='{.items[0].metadata.name}')
    if [ "$(kubectl get pods -n "$namespace" "$pod_name" -o jsonpath='{.status.containerStatuses[0].ready}')" == "true" ]; then
      status=$(kubectl logs "$pod_name" -n "$namespace" | grep -c "Waiting")
    fi
    sleep 1
  done
  # Copy backup files to pod
  copy_to_pod "$pod_name" "$arch" "/tmp/archive.tar"
  echo "Init" > start.txt
  copy_to_pod "$pod_name" "start.txt" "/tmp/start.txt"
  rm -f start.txt
  # Wait for the job to complete or fail using the loop with sleep
  while true; do
    local succeeded_status=$(kubectl logs "$pod_name" -n "$namespace" | grep -c "Script finished.")
    local failed_status=$(kubectl get job "$job_name" -n "$namespace" -o=jsonpath='{.status.failed}')
    
    if [ "$succeeded_status" = "1" ]; then
      echo "Job '$job_name' successful."
      kubectl logs "$pod_name" -n "$namespace" | grep -v "Waiting"
      kubectl delete -f ./tmp.yaml -n "$namespace" 2>&1
      kubectl delete -f "${job_path}/configmap.yaml" -n "$namespace" > /dev/null 2>&1
      rm -f ./tmp.yaml
      break
    elif [ "$failed_status" = "1" ]; then
      echo "Job '$job_name' failed. Displaying logs:"
      # Display the logs of the failed pod
      kubectl logs "$pod_name" -n "$namespace"
      kubectl delete -f ./tmp.yaml -n "$namespace" 2>&1
      kubectl delete -f "${job_path}/configmap.yaml" -n "$namespace" > /dev/null 2>&1
      rm -f ./tmp.yaml
      exit 1
    fi
    sleep 1
  done
}

# Function to restore services from backup
restore_services() {
  local redis_arch="${backup_path}/redis_${arch_postfix}"
  local influx_arch="${backup_path}/influxdb_${arch_postfix}"
  check_file_existence "$redis_arch"
  check_file_existence "$influx_arch"

  local redis_pod=$(find_statefulset_with_image "redis")
  local influx_pod=$(find_statefulset_with_image "influxdb")

  # Unpack backup archives and copy to redis pod
  rm -rf "${backup_path}"/redis-tmp || true
  mkdir "${backup_path}"/redis-tmp
  tar -C "${backup_path}"/redis-tmp -xf "$redis_arch"
  for file in $(ls ${backup_path}/redis-tmp); do
    local full_path="${backup_path}/redis-tmp/$file"
    kubectl cp "$full_path" "$redis_pod:/data/$file" -n "$namespace" --retries 999 --no-preserve
  done
  rm -rf "${backup_path}"/redis-tmp

  # Copy backup archives to influxdb pod and unpack
  kubectl cp "$influx_arch" "$influx_pod:/tmp/influxdb.tar" -n "$namespace" --retries 999 --no-preserve
  kubectl exec "$influx_pod" -n "$namespace" -- tar -C /var/lib/influxdb -xf /tmp/influxdb.tar
  kubectl exec "$influx_pod" -n "$namespace" -- rm -f /tmp/influxdb.tar

  echo "Data restored to 'auth-cache' and 'influxdb'."
}

restore_postgres() {
  local target_arch="${backup_path}/postgres_${arch_postfix}"
  check_file_existence "$target_arch"

  # Run restore job
  run_job "postgres-restore-job" "./jobs/postgres/restore" "$target_arch"

  echo "Postgres databases restored."
}

restore_minio() {
  local target_arch="${backup_path}/minio_${arch_postfix}"
  check_file_existence "$target_arch"

  # Run restore job
  run_job "minio-restore-job" "./jobs/minio/restore" "$target_arch"
  
  echo "MiniO restored."
}

main() {
  # Main script logic
  echo "Starting restore process for namespace '$namespace' from backup date '$backup_date'..."

  export arch_postfix="backup.tar"
  pwd_compatible
  export backup_path="${work_dir}/backups/${backup_date}"
  check_directory_existence "$backup_path"

  declare -A replicas
  export replicas
  # Stop deployments
  echo "Stopping TestIT..."
  local testit=("frontend" "webapi" "background-service" "rabbitmqconsumer" "auth" "avatars-api" "ldapwebapi" "license-service" "globalsearch-service")
  stop_deployments "${testit[@]}"

  # Restore services
  echo "Restoring services..."
  restore_services

  # Restore databases
  echo "Restoring databases..."
  restore_postgres

  # Restore minio
  echo "Restoring buckets..."
  restore_minio

  # Start deployments
  echo "Starting TestIT..."
  start_deployments "${testit[@]}"

  echo "Restore process completed successfully!"
  exit 0
}

check_required_variables "postgres_vars" "${postgres_vars[@]}"
check_required_variables "minio_vars" "${minio_vars[@]}"
command_check "kubectl"
command_check "tar"
validate_input "$@"
namespace="$1"
validate_namespace "$namespace"
backup_date="$2"

main "$@"
exit 0
