#!/bin/bash
# minio-backup.sh

# Configurable options
TARGET_DIR="/backup"

set -euo pipefail

sleep 10

# Get config files from the source
prepareMigration() {
  local source="$1"
  local target_dir="$2"
  mkdir -p "$target_dir"
  cd "$target_dir"
  mc admin config export "$source" > config.txt
  mc admin cluster bucket export "$source"
  mc admin cluster iam export "$source"
}

# Download buckets from the source
mirrorBuckets() {
  local source="$1"
  mkdir "${TARGET_DIR}/${source}" || true
  mc ls "$source" | awk '{print $5}' | while read bucket
  do
    if [ "$bucket" == "${TMS_BUCKET_NAME}/" ] || [ "$bucket" == "${AVATARS_AWS_BUCKET_NAME}/" ]; then
      echo "Migration is in process, please wait..."
      # Get files from the source bucket
      mc mirror --preserve "$source/$bucket" "${TARGET_DIR}/${source}" >> /dev/null

      # Check for success
      SOURCE_FILES_COUNT=$(mc ls --recursive "$source/$bucket" | wc -l)
      TARGET_FILES_COUNT=$(ls "$TARGET_DIR/$bucket" | wc -l)
      RED='\033[0;31m'
      GREEN='\033[0;32m'
      NC='\033[0m'
      if [ "$SOURCE_FILES_COUNT" == "$TARGET_FILES_COUNT" ]; then
        printf "$bucket bucket migrated ${GREEN}sucessfully${NC}! \nSource files count: $SOURCE_FILES_COUNT\nTransferred files count: $TARGET_FILES_COUNT\n";
      else
        printf "$bucket bucket migration ${RED}failed${NC}! \nSource files: $SOURCE_FILES_COUNT\nFiles transferred: $TARGET_FILES_COUNT\n";
        exit 1;
      fi
    else
      echo "Skipping bucket: $bucket"
    fi
  done
}

# The main function to execute migration
main() {
  # Set the MinIO Client alias using provided environment variables
  mc alias set testit "$AWS_CONNECTION_STRING" "$AWS_ACCESS_KEY" "$AWS_SECRET_KEY"
  mc alias set avatars "$AVATARS_CONNECTION_STRING" "$AVATARS_ACCESS_KEY" "$AVATARS_SECRET_KEY"
  
  prepareMigration "testit" "$TARGET_DIR/testit_configs"
  mirrorBuckets "testit"

  prepareMigration "avatars" "$TARGET_DIR/avatars_configs"
  mirrorBuckets "avatars"
  chown -R "$USER":"$USER" "$TARGET_DIR"
}

# Execute the main function and handle errors
if main; then
  echo "MiniO migration completed successfully."
  exit 0  # Exit with code 0 for success
else
  echo "Error: MinIO migration failed."
  exit 1  # Exit with code 1 for failure
fi